@@ -9,12 +9,14 @@ const dataFilePath = core.getInput('dataFile');
 const CcAddresses = JSON.parse(core.getInput('ccAddresses'));
 const ToAddresses = JSON.parse(core.getInput('toAddresses'));
 const ReplyToAddresses = JSON.parse(core.getInput('replyToAddresses'));
+const workflowURL = core.getInput('workflowURL');
 
-const data = fs.readFileSync(dataFilePath, { encoding: 'utf-8' });
+const data = dataFilePath ? fs.readFileSync(dataFilePath, { encoding: 'utf-8' }) : "";
 
 const templated = {
     version,
     results: data,
+    workflowURL,
 };
 
 // Set the region
