@@ -238,6 +238,16 @@ func (cmd CatCmd) prettyPrintResults(ctx context.Context, doltSch schema.Schema,
 				}
 				if val != types.NullValue {
 					tag := uint64(tagVal.(types.Uint))
+
+					// TODO: This is a bit of hack but enough to last for a long time. This entire function
+					// needs to be rewritten to deal with a storage engine change. In the case of a keyless row
+					// we don't care about the row id tag or cardinality tag. Note, could just convert this into a row.FromNoms
+					// but keyless tables with unique indexes will get identified as keyed rows causing additional problems
+					// in the future. Broadly, in the future we won't need this when all storage engine changes are made.
+					if tag == schema.KeylessRowIdTag || tag == schema.KeylessRowCardinalityTag {
+						continue
+					}
+
 					strPtr, err := doltSch.GetAllCols().TagToCol[tag].TypeInfo.FormatValue(val)
 					if err != nil {
 						return nil, err
