@@ -19,6 +19,11 @@ package serial
 const StoreRootFileID = "STRT"
 const TagFileID = "DTAG"
 const WorkingSetFileID = "WRST"
+const CommitFileID = "DCMT"
+const RootValueFileID = "RTVL"
+const TableFileID = "DTBL"
+const ProllyTreeNodeFileID = "TUPM"
+const AddressMapFileID = "ADRM"
 
 func GetFileID(bs []byte) string {
 	if len(bs) < 8 {
