@@ -16,9 +16,9 @@ package doltdb
 
 import (
 	"context"
-	"errors"
 	"io"
 
+	"github.com/dolthub/dolt/go/store/datas"
 	"github.com/dolthub/dolt/go/store/hash"
 	"github.com/dolthub/dolt/go/store/types"
 )
@@ -142,17 +142,11 @@ func (cmItr *commitItr) Next(ctx context.Context) (hash.Hash, *Commit, error) {
 }
 
 func hashToCommit(ctx context.Context, vrw types.ValueReadWriter, h hash.Hash) (*Commit, error) {
-	val, err := vrw.ReadValue(ctx, h)
-
+	dc, err := datas.LoadCommitAddr(ctx, vrw, h)
 	if err != nil {
 		return nil, err
 	}
-
-	if val == nil {
-		return nil, errors.New("failed to get commit")
-	}
-
-	return NewCommit(ctx, vrw, val)
+	return NewCommit(ctx, vrw, dc)
 }
 
 // CommitFilter is a function that returns true if a commit should be filtered out, and false if it should be kept
