@@ -28,11 +28,20 @@ import (
 
 const DoltResetFuncName = "dolt_reset"
 
+// Deprecated: please use the version in the dprocedures package
 type DoltResetFunc struct {
 	children []sql.Expression
 }
 
 func (d DoltResetFunc) Eval(ctx *sql.Context, row sql.Row) (interface{}, error) {
+	args, err := getDoltArgs(ctx, row, d.Children())
+	if err != nil {
+		return 1, err
+	}
+	return DoDoltReset(ctx, args)
+}
+
+func DoDoltReset(ctx *sql.Context, args []string) (int, error) {
 	dbName := ctx.GetCurrentDatabase()
 
 	if len(dbName) == 0 {
@@ -46,14 +55,7 @@ func (d DoltResetFunc) Eval(ctx *sql.Context, row sql.Row) (interface{}, error)
 		return 1, fmt.Errorf("Could not load database %s", dbName)
 	}
 
-	ap := cli.CreateResetArgParser()
-	args, err := getDoltArgs(ctx, row, d.Children())
-
-	if err != nil {
-		return 1, err
-	}
-
-	apr, err := ap.Parse(args)
+	apr, err := cli.CreateResetArgParser().Parse(args)
 	if err != nil {
 		return 1, err
 	}
@@ -93,7 +95,7 @@ func (d DoltResetFunc) Eval(ctx *sql.Context, row sql.Row) (interface{}, error)
 
 		ws, err := dSess.WorkingSet(ctx, dbName)
 		if err != nil {
-			return nil, err
+			return 1, err
 		}
 		err = dSess.SetWorkingSet(ctx, dbName, ws.WithWorkingRoot(roots.Working).WithStagedRoot(roots.Staged))
 		if err != nil {
@@ -154,6 +156,7 @@ func (d DoltResetFunc) WithChildren(children ...sql.Expression) (sql.Expression,
 	return NewDoltResetFunc(children...)
 }
 
+// Deprecated: please use the version in the dprocedures package
 func NewDoltResetFunc(args ...sql.Expression) (sql.Expression, error) {
 	return DoltResetFunc{children: args}, nil
 }
