@@ -130,7 +130,7 @@ func PutField(tb *val.TupleBuilder, i int, v interface{}) error {
 	case val.GeometryEnc:
 		tb.PutGeometry(i, serializeGeometry(v))
 	case val.JSONEnc:
-		buf, err := json.Marshal(v.(sql.JSONDocument).Val)
+		buf, err := convJson(v)
 		if err != nil {
 			return err
 		}
@@ -193,6 +193,14 @@ func convUint(v interface{}) uint {
 	}
 }
 
+func convJson(v interface{}) (buf []byte, err error) {
+	v, err = sql.JSON.Convert(v)
+	if err != nil {
+		return nil, err
+	}
+	return json.Marshal(v.(sql.JSONDocument).Val)
+}
+
 func deserializeGeometry(buf []byte) (v interface{}) {
 	srid, _, typ := geo.ParseEWKBHeader(buf)
 	buf = buf[geo.EWKBHeaderSize:]
