@@ -88,7 +88,7 @@ func LoadedLocalLocation() *time.Location {
 func BasicSelectTests() []SelectTest {
 	headCommitHash := "73hc2robs4v0kt9taoe3m5hd49dmrgun"
 	if types.Format_Default == types.Format_DOLT_DEV {
-		headCommitHash = "oc0ho5dpdfr6tvdb3kvctphllsct46ir"
+		headCommitHash = "8hrmmj29n9keblqs7r9423f69uesfgn6"
 	}
 	return []SelectTest{
 		{
