@@ -28,6 +28,7 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/dtestutils"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dtables"
+	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/json"
 	"github.com/dolthub/dolt/go/store/types"
 )
 
@@ -408,11 +409,12 @@ var systemTableUpdateTests = []UpdateTest{
 				types.String("name"),
 				types.String("select 2+2 from dual"),
 				types.Int(1),
+				CreateTestJSON(),
 			)),
 		UpdateQuery: "update dolt_schemas set type = 'not a view'",
 		SelectQuery: "select * from dolt_schemas",
 		ExpectedRows: ToSqlRows(CompressSchema(SchemasTableSchema()),
-			NewRow(types.String("not a view"), types.String("name"), types.String("select 2+2 from dual"), types.Int(1)),
+			NewRow(types.String("not a view"), types.String("name"), types.String("select 2+2 from dual"), types.Int(1), CreateTestJSON()),
 		),
 		ExpectedSchema: CompressSchema(SchemasTableSchema()),
 	},
@@ -449,7 +451,21 @@ func testUpdateQuery(t *testing.T, test UpdateTest) {
 	actualRows, sch, err := executeSelect(t, context.Background(), dEnv, root, test.SelectQuery)
 	require.NoError(t, err)
 
-	assert.Equal(t, test.ExpectedRows, actualRows)
+	assert.Equal(t, len(test.ExpectedRows), len(actualRows))
+	for i := 0; i < len(test.ExpectedRows); i++ {
+		assert.Equal(t, len(test.ExpectedRows[i]), len(actualRows[i]))
+		for j := 0; j < len(test.ExpectedRows[i]); j++ {
+			if _, ok := actualRows[i][j].(json.NomsJSON); ok {
+				cmp, err := actualRows[i][j].(json.NomsJSON).Compare(nil, test.ExpectedRows[i][j].(json.NomsJSON))
+				assert.NoError(t, err)
+				assert.Equal(t, 0, cmp)
+			} else {
+				assert.Equal(t, test.ExpectedRows[i][j], actualRows[i][j])
+			}
+
+		}
+	}
+
 	sqlSchema := mustSqlSchema(test.ExpectedSchema)
 	assertSchemasEqual(t, sqlSchema, sch)
 }
