@@ -95,6 +95,24 @@ func (w *SqlExportWriter) WriteSqlRow(ctx context.Context, r sql.Row) error {
 		return nil
 	}
 
+	// Special case for schemas table
+	if w.tableName == doltdb.SchemasTableName {
+		stmt, err := sqlfmt.SqlRowAsCreateFragStmt(r)
+		if err != nil {
+			return err
+		}
+		return iohelp.WriteLine(w.wr, stmt)
+	}
+
+	// Special case for procedures table
+	if w.tableName == doltdb.ProceduresTableName {
+		stmt, err := sqlfmt.SqlRowAsCreateProcStmt(r)
+		if err != nil {
+			return err
+		}
+		return iohelp.WriteLine(w.wr, stmt)
+	}
+
 	if err := w.maybeWriteDropCreate(ctx); err != nil {
 		return err
 	}
@@ -108,6 +126,10 @@ func (w *SqlExportWriter) WriteSqlRow(ctx context.Context, r sql.Row) error {
 }
 
 func (w *SqlExportWriter) maybeWriteDropCreate(ctx context.Context) error {
+	// Never write create table for DoltSchemasTable
+	if w.tableName == doltdb.SchemasTableName || w.tableName == doltdb.ProceduresTableName {
+		return nil
+	}
 	if !w.writtenFirstRow {
 		var b strings.Builder
 		b.WriteString(sqlfmt.DropTableIfExistsStmt(w.tableName))
