@@ -673,9 +673,9 @@ func TestFindCommonAncestor(t *testing.T) {
 
 	assertCommonAncestor(t, a6, a9, ra9, db, rdb) // Common third parent
 
-	a9c, err := CommitFromValue(db.Format(), a9)
+	a9c, err := commitFromValue(db.Format(), a9)
 	require.NoError(t, err)
-	ra9c, err := CommitFromValue(rdb.Format(), ra9)
+	ra9c, err := commitFromValue(rdb.Format(), ra9)
 	require.NoError(t, err)
 	_, _, err = FindCommonAncestor(context.Background(), ra9c, a9c, db, rdb)
 	assert.Error(err)
