@@ -218,7 +218,7 @@ func (suite *DatabaseSuite) TestDatabaseCommit() {
 	suite.NoError(err)
 	suite.True(ok)
 	suite.True(h.Equals(a))
-	comm, err := CommitFromValue(suite.db.Format(), mustHead(ds2))
+	comm, err := commitFromValue(suite.db.Format(), mustHead(ds2))
 	suite.NoError(err)
 	suite.Equal(uint64(1), comm.Height())
 
@@ -230,7 +230,7 @@ func (suite *DatabaseSuite) TestDatabaseCommit() {
 	ds, err = CommitValue(context.Background(), suite.db, ds, b)
 	suite.NoError(err)
 	suite.True(mustHeadValue(ds).Equals(b))
-	comm, err = CommitFromValue(suite.db.Format(), mustHead(ds))
+	comm, err = commitFromValue(suite.db.Format(), mustHead(ds))
 	suite.NoError(err)
 	suite.Equal(uint64(2), comm.Height())
 
@@ -247,7 +247,7 @@ func (suite *DatabaseSuite) TestDatabaseCommit() {
 	ds, err = CommitValue(context.Background(), suite.db, ds, d)
 	suite.NoError(err)
 	suite.True(mustHeadValue(ds).Equals(d))
-	comm, err = CommitFromValue(suite.db.Format(), mustHead(ds))
+	comm, err = commitFromValue(suite.db.Format(), mustHead(ds))
 	suite.NoError(err)
 	suite.Equal(uint64(3), comm.Height())
 
