@@ -129,8 +129,18 @@ func (sm SerialMessage) walkRefs(nbf *NomsBinFormat, cb RefCallback) error {
 				return err
 			}
 		}
-		if msg.MergeStateAddrLength() != 0 {
-			addr = hash.New(msg.MergeStateAddrBytes())
+		mergeState := msg.MergeState(nil)
+		if mergeState != nil {
+			addr = hash.New(mergeState.PreWorkingRootAddrBytes())
+			r, err = constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+
+			addr = hash.New(mergeState.FromCommitAddrBytes())
 			r, err = constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
 			if err != nil {
 				return err
@@ -139,12 +149,142 @@ func (sm SerialMessage) walkRefs(nbf *NomsBinFormat, cb RefCallback) error {
 				return err
 			}
 		}
-	case serial.CommitFileID:
-		parents, err := SerialCommitParentRefs(nbf, sm)
+	case serial.RootValueFileID:
+		msg := serial.GetRootAsRootValue([]byte(sm), 0)
+		rm := msg.Tables(nil)
+		refs := rm.RefArrayBytes()
+		for i := 0; i < rm.NamesLength(); i++ {
+			off := i * 20
+			addr := hash.New(refs[off : off+20])
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+		}
+		addr := hash.New(msg.ForeignKeyAddrBytes())
+		if !addr.IsEmpty() {
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+		}
+		addr = hash.New(msg.SuperSchemasAddrBytes())
+		if !addr.IsEmpty() {
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+		}
+	case serial.TableFileID:
+		msg := serial.GetRootAsTable([]byte(sm), 0)
+		addr := hash.New(msg.SchemaBytes())
+		r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
 		if err != nil {
 			return err
 		}
-		for _, r := range parents {
+		err = cb(r)
+		if err != nil {
+			return err
+		}
+
+		addr = hash.New(msg.ViolationsBytes())
+		if !addr.IsEmpty() {
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+		}
+
+		confs := msg.Conflicts(nil)
+		addr = hash.New(confs.DataBytes())
+		if !addr.IsEmpty() {
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+		}
+
+		addr = hash.New(confs.OurSchemaBytes())
+		if !addr.IsEmpty() {
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+		}
+
+		addr = hash.New(confs.TheirSchemaBytes())
+		if !addr.IsEmpty() {
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+		}
+
+		addr = hash.New(confs.AncestorSchemaBytes())
+		if !addr.IsEmpty() {
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+		}
+
+		rm := msg.SecondaryIndexes(nil)
+		refs := rm.RefArrayBytes()
+		for i := 0; i < rm.NamesLength(); i++ {
+			off := i * 20
+			addr := hash.New(refs[off : off+20])
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
+			if err = cb(r); err != nil {
+				return err
+			}
+		}
+
+		mapbytes := msg.PrimaryIndexBytes()
+
+		dec := newValueDecoder(mapbytes, nil)
+		v, err := dec.readValue(nbf)
+		if err != nil {
+			return err
+		}
+
+		return v.walkRefs(nbf, cb)
+	case serial.CommitFileID:
+		parents, err := SerialCommitParentAddrs(nbf, sm)
+		if err != nil {
+			return err
+		}
+		for _, addr := range parents {
+			r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
+			if err != nil {
+				return err
+			}
 			if err = cb(r); err != nil {
 				return err
 			}
@@ -163,19 +303,15 @@ func (sm SerialMessage) walkRefs(nbf *NomsBinFormat, cb RefCallback) error {
 	return nil
 }
 
-func SerialCommitParentRefs(nbf *NomsBinFormat, sm SerialMessage) ([]Ref, error) {
+func SerialCommitParentAddrs(nbf *NomsBinFormat, sm SerialMessage) ([]hash.Hash, error) {
 	msg := serial.GetRootAsCommit([]byte(sm), 0)
 	addrs := msg.ParentAddrsBytes()
 	n := len(addrs) / 20
-	ret := make([]Ref, n)
+	ret := make([]hash.Hash, n)
 	for i := 0; i < n; i++ {
 		addr := hash.New(addrs[:20])
 		addrs = addrs[20:]
-		r, err := constructRef(nbf, addr, PrimitiveTypeMap[ValueKind], SerialMessageRefHeight)
-		if err != nil {
-			return nil, err
-		}
-		ret[i] = r
+		ret[i] = addr
 	}
 	return ret, nil
 }
