@@ -123,7 +123,7 @@ func (t *Type) WalkValues(ctx context.Context, cb ValueCallback) error {
 	return t.Desc.walkValues(cb)
 }
 
-func (t *Type) WalkRefs(nbf *NomsBinFormat, cb RefCallback) error {
+func (t *Type) walkRefs(nbf *NomsBinFormat, cb RefCallback) error {
 	return nil
 }
 
